@@ -1,5 +1,7 @@
 """Utility function for variable selection and bart interpretability."""
 
+import warnings
+
 import arviz as az
 import matplotlib.pyplot as plt
 import numpy as np
@@ -7,7 +9,7 @@ import numpy as np
 from pytensor.tensor.var import Variable
 from scipy.interpolate import griddata
 from scipy.signal import savgol_filter
-from scipy.stats import pearsonr
+from scipy.stats import pearsonr, norm
 
 
 def _sample_posterior(all_trees, X, rng, size=None, excluded=None):
@@ -47,11 +49,67 @@ def _sample_posterior(all_trees, X, rng, size=None, excluded=None):
 
     for ind, p in enumerate(pred):
         for tree in stacked_trees[idx[ind]]:
-            p += np.array([tree.predict(x, excluded) for x in X])
+            p += np.vstack([tree.predict(x, excluded) for x in X])
     pred.reshape((*size, shape, -1))
     return pred
 
 
+def plot_convergence(idata, var_name=None, kind="ecdf", figsize=None, ax=None):
+    """
+    Plot convergence diagnostics.
+
+    Parameters
+    ----------
+    idata : InferenceData
+        InferenceData object containing the posterior samples.
+    var_name : str
+        Name of the BART variable to plot. Defaults to None.
+    kind : str
+        Type of plot to display. Options are "ecdf" (default) and "kde".
+    figsize : tuple
+        Figure size. Defaults to None.
+    ax : matplotlib axes
+        Axes on which to plot. Defaults to None.
+
+    Returns
+    -------
+    ax : matplotlib axes
+    """
+    ess_threshold = idata.posterior.chain.size * 100
+    ess = np.atleast_2d(az.ess(idata, method="bulk", var_names=var_name)[var_name].values)
+    rhat = np.atleast_2d(az.rhat(idata, var_names=var_name)[var_name].values)
+
+    if figsize is None:
+        figsize = (10, 3)
+
+    if kind == "ecdf":
+        kind_func = az.plot_ecdf
+        sharey = True
+    elif kind == "kde":
+        kind_func = az.plot_kde
+        sharey = False
+
+    if ax is None:
+        _, ax = plt.subplots(1, 2, figsize=figsize, sharex="col", sharey=sharey)
+
+    for idx, (essi, rhati) in enumerate(zip(ess, rhat)):
+        kind_func(essi, ax=ax[0], plot_kwargs={"color": f"C{idx}"})
+        kind_func(rhati, ax=ax[1], plot_kwargs={"color": f"C{idx}"})
+
+    ax[0].axvline(ess_threshold, color="0.7", ls="--")
+    # Assume Rhats are N(1, 0.005) iid. Then compute the 0.99 quantile
+    # scaled by the sample size and use it as a threshold.
+    ax[1].axvline(norm(1, 0.005).ppf(0.99 ** (1 / ess.size)), color="0.7", ls="--")
+
+    ax[0].set_xlabel("ESS")
+    ax[1].set_xlabel("R-hat")
+    if kind == "kde":
+        ax[0].set_yticks([])
+        ax[1].set_yticks([])
+
+    return ax
+
+
 def plot_dependence(
     bartrv,
     X,
@@ -61,6 +119,7 @@ def plot_dependence(
     xs_values=None,
     var_idx=None,
     var_discrete=None,
+    func=None,
     samples=50,
     instances=10,
     random_seed=None,
@@ -104,6 +163,8 @@ def plot_dependence(
         List of the indices of the covariate for which to compute the pdp or ice.
     var_discrete : list
         List of the indices of the covariate treated as discrete.
+    func : function
+        Arbitrary function to apply to the predictions. Defaults to the identity function.
     samples : int
         Number of posterior samples used in the predictions. Defaults to 50
     instances : int
@@ -228,7 +289,10 @@ def plot_dependence(
         y_mins.append(np.min(y_pred))
         new_y.append(np.array(y_pred).T)
 
+    if func is not None:
+        new_y = [func(nyi) for nyi in new_y]
     shape = 1
+    new_y = np.array(new_y)
     if new_y[0].ndim == 3:
         shape = new_y[0].shape[0]
     if ax is None:
@@ -238,6 +302,14 @@ def plot_dependence(
             fig, axes = plt.subplots(1, len(var_idx) * shape, sharey=sharey, figsize=figsize)
         elif isinstance(grid, tuple):
             fig, axes = plt.subplots(grid[0], grid[1], sharey=sharey, figsize=figsize)
+            grid_size = grid[0] * grid[1]
+            n_plots = new_y.squeeze().shape[0]
+            if n_plots > grid_size:
+                warnings.warn("The grid is smaller than the number of available variables to plot")
+            elif n_plots < grid_size:
+                for i in range(n_plots, grid[0] * grid[1]):
+                    fig.delaxes(axes.flatten()[i])
+                axes = axes.flatten()[:n_plots]
         axes = np.ravel(axes)
     else:
         axes = [ax]
@@ -246,10 +318,6 @@ def plot_dependence(
     x_idx = 0
     y_idx = 0
     for ax in axes:  # pylint: disable=redefined-argument-from-local
-        if x_idx >= len(var_idx):
-            ax.set_axis_off()
-            fig.delaxes(ax)
-
         nyi = new_y[x_idx][y_idx]
         nxi = new_x_target[x_idx]
         var = var_idx[x_idx]
@@ -396,6 +464,7 @@ def plot_variable_importance(
 
     axes[1].errorbar(ticks, ev_mean, np.array((ev_mean - ev_hdi[:, 0], ev_hdi[:, 1] - ev_mean)))
 
+    axes[1].axhline(ev_mean[-1], ls="--", color="0.5")
     axes[1].set_xticks(ticks)
     axes[1].set_xticklabels(ticks + 1)
     axes[1].set_xlabel("number of covariables")
